@@ -5,6 +5,7 @@
 '''Zim test suite'''
 
 import os
+import sys
 import shutil
 import unittest
 import gettext
@@ -38,8 +39,12 @@ def set_environ():
 		'XDG_CONFIG_DIRS': './tests/tmp/config_dir',
 		'XDG_CACHE_HOME': './tests/tmp/cache_home'
 	})
+	if os.name == 'nt':
+		tmpdir = unicode(tmpdir)
+	else:
+		tmpdir = tmpdir.encode(sys.getfilesystemencoding())
 	if os.path.isdir(tmpdir):
-		shutil.rmtree(unicode(tmpdir))
+		shutil.rmtree(tmpdir)
 	os.mkdir(tmpdir)
 
 
@@ -49,11 +54,11 @@ def create_tmp_dir(name):
 	is called.
 	'''
 	dir = os.path.join('tests', 'tmp', name)
+	if os.name == 'nt':
+		dir = unicode(dir)
+	else:
+		dir = dir.encode(sys.getfilesystemencoding())
 	if os.path.exists(dir):
-		# Make sure to hand unicode to rmtree - looks like it filters
-		# through to listdir somewhere...
-		if not isinstance(dir, unicode):
-			dir = dir.decode('utf-8')
 		shutil.rmtree(dir)
 	assert not os.path.exists(dir) # make real sure
 	os.makedirs(dir)
@@ -103,9 +108,10 @@ def get_test_notebook(format='wiki'):
 	store = notebook.add_store(Path(':'), 'memory')
 	manifest = []
 	for name, text in get_test_data(format):
-			manifest.append(name)
-			store.set_node(Path(name), text)
+		manifest.append(name)
+		store.set_node(Path(name), text)
 	notebook.testdata_manifest = expand_manifest(manifest)
+	notebook.index.update()
 	return notebook
 
 
@@ -131,6 +137,16 @@ def get_test_page(name='Foo'):
 	return notebook, notebook.get_page(Path(name))
 
 
+def print_index(index):
+	print '==== INDEX ===='
+	for page in index.walk():
+		print page.name, page.hascontent, page.haschildren
+		for link in index.list_links(page):
+			print '\t->', link.href.name
+	print '==============='
+
+
+
 class TestCase(unittest.TestCase):
 	'''FIXME'''
 
