@@ -18,14 +18,14 @@ class TestDirsTestSetup(TestCase):
 	def runTest(self):
 		'''Test config environment setup of test'''
 		for k, v in (
-			('XDG_DATA_HOME', './tests/tmp/share'),
-			('XDG_CONFIG_HOME', './tests/tmp/config'),
-			('XDG_CACHE_HOME', './tests/tmp/cache')
+			('XDG_DATA_HOME', './tests/tmp/data_home'),
+			('XDG_CONFIG_HOME', './tests/tmp/config_home'),
+			('XDG_CACHE_HOME', './tests/tmp/cache_home')
 		): self.assertEqual(getattr(zim.config, k), Dir(v))
 
 		for k, v in (
-			('XDG_DATA_DIRS', './tests/tmp/share'),
-			('XDG_CONFIG_DIRS', './tests/tmp/config'),
+			('XDG_DATA_DIRS', './tests/tmp/data_dir'),
+			('XDG_CONFIG_DIRS', './tests/tmp/config_dir'),
 		): self.assertEqual(getattr(zim.config, k), map(Dir, v.split(':')))
 
 
@@ -172,6 +172,7 @@ none=None
 
 	def testLookup(self):
 		'''Test lookup of config files'''
+		XDG_CONFIG_DIRS[0].file('zim/preferences.conf').touch()
 		file = config_file('preferences.conf')
 		self.assertTrue(isinstance(file, ConfigDictFile))
 		self.assertTrue(file.default.exists())
