@@ -116,11 +116,12 @@ class CustomToolList(gtk.TreeView):
 		self.get_selection.select_path(path)
 
 	def refresh(self):
+		from zim.gui.widgets import _encode_xml
 		model = self.get_model()
 		model.clear()
 		for tool in self.manager:
-			pixbuf = tool.get_pixbuf(gtk.ICON_SIZE_BUTTON)
-			text = '<b>%s</b>\n%s' % (tool.name, tool.comment)
+			pixbuf = tool.get_pixbuf(gtk.ICON_SIZE_MENU)
+			text = '<b>%s</b>\n%s' % (_encode_xml(tool.name), _encode_xml(tool.comment))
 			model.append((pixbuf, text, tool.key))
 
 
@@ -152,7 +153,11 @@ class EditCustomToolDialog(Dialog):
 		), trigger_response=False)
 
 		# FIXME need ui builder to take care of this as well
-		self.iconbutton = IconChooserButton(stock=gtk.STOCK_EXECUTE)
+		if tool:
+			iconpixbuf = tool.get_pixbuf(gtk.ICON_SIZE_DIALOG)
+		else:
+			iconpixbuf = None
+		self.iconbutton = IconChooserButton(stock=gtk.STOCK_EXECUTE, pixbuf=iconpixbuf)
 		label = gtk.Label(_('Icon')+':') # T: Input in "Edit Custom Tool" dialog
 		label.set_alignment(0.0, 0.5)
 		hbox = gtk.HBox()
@@ -181,6 +186,6 @@ in the command when it is executed:
 
 	def do_response_ok(self):
 		fields = self.get_fields()
-		fields['icon'] = self.iconbutton.get_file()
+		fields['Icon'] = self.iconbutton.get_file().path
 		self.result = fields
 		return True
