@@ -157,9 +157,11 @@ def resolve_notebook(string):
 
 	page = None
 	if is_url_re.match(string):
+		if string.startswith('zim+'): string = string[4:]
 		assert string.startswith('file://')
 		if '?' in string:
 			filepath, page = string.split('?', 1)
+			page = Path(page)
 		else:
 			filepath = string
 	elif os.path.sep in string:
@@ -168,12 +170,15 @@ def resolve_notebook(string):
 		nblist = get_notebook_list()
 		filepath = nblist.get_by_name(string)
 		if filepath is None:
-			return None, None # not found
+			if os.path.exists(string):
+				filepath = string # fall back to file path
+			else:
+				return None, None # not found
 
 	file = File(filepath) # Fixme need generic FS Path object here
 	if filepath.endswith('notebook.zim'):
 		return File(filepath).dir, page
-	elif file.exists(): # file exists and really is a file
+	elif not page and file.exists(): # file exists and really is a file
 		parents = list(file)
 		parents.reverse()
 		for parent in parents:
@@ -258,9 +263,9 @@ def interwiki_link(link):
 			break
 	else:
 		list = get_notebook_list()
-		for name, path in list.get_names():
+		for name, uri in list.get_names():
 			if name.lower() == key.lower():
-				url = path + '?{NAME}'
+				url = 'zim+' + uri + '?{NAME}'
 				break
 
 	if url and is_url_re.match(url):
@@ -616,6 +621,11 @@ class Notebook(gobject.GObject):
 			parent = source.commonparent(href)
 			if parent.isroot:
 				return ':' + href.name
+			elif parent == source.parent:
+				if parent == href:
+					return href.basename
+				else:
+					return href.relname(parent)
 			else:
 				return parent.basename + ':' + href.relname(parent)
 
@@ -649,10 +659,13 @@ class Notebook(gobject.GObject):
 			return None
 
 	@staticmethod
-	def cleanup_pathname(name):
+	def cleanup_pathname(name, purge=False):
 		'''Returns a safe version of name, used internally by functions like
 		resolve_path() to parse user input.
 		It raises a PageNameError when the name is not valid
+
+		If 'purge' is True any invalid characters will be removed,
+		otherwise they will result in a PageNameError exception.
 		'''
 		# Reserved characters are:
 		# The ':' is reserrved as seperator
@@ -668,6 +681,8 @@ class Notebook(gobject.GObject):
 		# For file system we should reserve (win32 & posix)
 		# "\", "/", ":", "*", "?", '"', "<", ">", "|"
 
+		# Do not allow '\n' for obvious reasons
+
 		# Allowing '%' will cause problems with sql wildcards sooner
 		# or later - also for url decoding ambiguity it is better to
 		# keep this one reserved
@@ -677,9 +692,13 @@ class Notebook(gobject.GObject):
 			# Avoid duplicates with and without '_' (e.g. in index)
 			# Note that leading "_" is stripped, due to strip() below
 
-		for char in ("?", "#", "/", "\\", "*", '"', "<", ">", "|", "%"):
-			if char in name:
-				raise PageNameError, orig
+		if purge:
+			for char in ("?", "#", "/", "\\", "*", '"', "<", ">", "|", "%", "\n"):
+				name = name.replace(char, '')
+		else:
+			for char in ("?", "#", "/", "\\", "*", '"', "<", ">", "|", "%", "\n"):
+				if char in name:
+					raise PageNameError, orig
 
 		parts = map(unicode.strip, filter(
 			lambda n: len(n)>0, unicode(name).split(':') ) )
@@ -742,10 +761,29 @@ class Notebook(gobject.GObject):
 		else:
 			store = self.get_store(path)
 			page = store.get_page(path)
+			indexpath = self.index.lookup_path(path)
+			if indexpath and indexpath.haschildren:
+				page.haschildren = True
+				# page might be the parent of a placeholder, in that case
+				# the index knows it has children, but the store does not
+
 			# TODO - set haschildren if page maps to a store namespace
 			self._page_cache[path.name] = page
 			return page
 
+	def get_new_page(self, path):
+		'''Like get_page() but guarantees the page does not yet exist.
+		Will add a number to make name unique.
+		'''
+		i = 0
+		base = path.name
+		page = self.get_page(path)
+		while page.hascontent or page.haschildren:
+			i += 1
+			path = Path(base + ' %i' % i)
+			page = self.get_page(path)
+		return page
+
 	def flush_page_cache(self, path):
 		'''Remove a page from the page cache, calling get_page() after this
 		will return a fresh page object. Be aware that the old object
@@ -827,42 +865,52 @@ class Notebook(gobject.GObject):
 		page.set_parsetree(storedpage.get_parsetree())
 		page.modified = False
 
-	def move_page(self, path, newpath, update_links=True):
+	def move_page(self, path, newpath, update_links=True, callback=None):
 		'''Move a page from 'path' to 'newpath'. If 'update_links' is
 		True all links from and to the page will be modified as well.
+		The original page 'path' does not have to exist, this is usefull
+		to update links for a placeholder. If 'newpath' exists a
+		PageExistsError error will be raised.
 		'''
+		if path == newpath:
+			return
+
 		if update_links and self.index.updating:
 			raise IndexBusyError, 'Index busy'
 			# Index need to be complete in order to be 100% sure we
 			# know all backlinks, so no way we can update links before.
 
 		page = self.get_page(path)
-		if not (page.hascontent or page.haschildren):
-			raise LookupError, 'Page does not exist: %s' % path.name
 		assert not page.modified, 'BUG: moving a page with uncomitted changes'
 
+		newpage = self.get_page(newpath)
+		if newpage.exists():
+			raise PageExistsError, 'Page already exists: %s' % newpath.name
+
 		self.emit('move-page', path, newpath, update_links)
 		logger.debug('Move %s to %s (%s)', path, newpath, update_links)
 
 		# Collect backlinks
 		if update_links:
 			from zim.index import LINK_DIR_BACKWARD
-			backlinkpages = set(
-				l.source for l in
-					self.index.list_links(path, LINK_DIR_BACKWARD) )
-			for child in self.index.walk(path):
-				backlinkpages.update(set(
-					l.source for l in
-						self.index.list_links(path, LINK_DIR_BACKWARD) ))
+			backlinkpages = set()
+			for l in self.index.list_links(path, LINK_DIR_BACKWARD):
+				backlinkpages.add(l.source)
 
-		# Do the actual move
-		store = self.get_store(path)
-		newstore = self.get_store(newpath)
-		if newstore == store:
-			store.move_page(path, newpath)
-		else:
-			assert False, 'TODO: move between stores'
-			# recursive + move attachments as well
+			if page.haschildren:
+				for child in self.index.walk(path):
+					for l in self.index.list_links(child, LINK_DIR_BACKWARD):
+						backlinkpages.add(l.source)
+
+		# Do the actual move (if the page exists)
+		if page.exists():
+			store = self.get_store(path)
+			newstore = self.get_store(newpath)
+			if newstore == store:
+				store.move_page(path, newpath)
+			else:
+				assert False, 'TODO: move between stores'
+				# recursive + move attachments as well
 
 		self.flush_page_cache(path)
 		self.flush_page_cache(newpath)
@@ -870,15 +918,17 @@ class Notebook(gobject.GObject):
 		# Update links in moved pages
 		page = self.get_page(newpath)
 		if page.hascontent:
-			self._update_links_from(page, path)
+			if callback: callback(page)
+			self._update_links_from(page, path, page, path)
 			store = self.get_store(page)
 			store.store_page(page)
 			# do not use self.store_page because it emits signals
 		for child in self._no_index_walk(newpath):
 			if not child.hascontent:
 				continue
+			if callback: callback(child)
 			oldpath = path + child.relname(newpath)
-			self._update_links_from(child, oldpath)
+			self._update_links_from(child, oldpath, newpath, path)
 			store = self.get_store(child)
 			store.store_page(child)
 			# do not use self.store_page because it emits signals
@@ -889,10 +939,12 @@ class Notebook(gobject.GObject):
 			self.index.delete(path)
 			self.index.update(newpath)
 			#~ print backlinkpages
+			total = len(backlinkpages)
 			for p in backlinkpages:
 				if p == path or p.ischild(path):
 					continue
 				page = self.get_page(p)
+				if callback: callback(page, total=total)
 				self._update_links_in_page(page, path, newpath)
 				self.store_page(page)
 
@@ -915,7 +967,7 @@ class Notebook(gobject.GObject):
 			tag.text = newhref
 		tag.attrib['href'] = newhref
 
-	def _update_links_from(self, page, oldpath):
+	def _update_links_from(self, page, oldpath, parent, oldparent):
 		logger.debug('Updating links in %s (was %s)', page, oldpath)
 		tree = page.get_parsetree()
 		if not tree:
@@ -938,6 +990,17 @@ class Notebook(gobject.GObject):
 							newhref = self.relative_link(page, oldhrefpath)
 							#~ print '\t->', newhref
 							self._update_link_tag(tag, newhref)
+					elif (hrefpath == oldparent or hrefpath.ischild(oldparent)):
+						# Special case where we e.g. link to our own children using
+						# a common parent between old and new path as an anchor for resolving
+						newhrefpath = parent
+						if hrefpath.ischild(oldparent):
+							newhrefpath = parent + hrefpath.relname(oldparent)
+						newhref = self.relative_link(page, newhrefpath)
+						#~ print '\t->', newhref
+						self._update_link_tag(tag, newhref)
+					else:
+						pass
 			except:
 				logger.exception('Error while updating link "%s"', href)
 
@@ -978,8 +1041,7 @@ class Notebook(gobject.GObject):
 
 		page.set_parsetree(tree)
 
-	def rename_page(self, path, newbasename,
-						update_heading=True, update_links=True):
+	def rename_page(self, path, newbasename, update_heading=True, update_links=True, callback=None):
 		'''Rename page to a page in the same namespace but with a new
 		basename. If 'update_heading' is True the first heading in the
 		page will be updated to it's new name.  If 'update_links' is
@@ -995,7 +1057,7 @@ class Notebook(gobject.GObject):
 			newpath = self.index.resolve_case(
 				newbasename, namespace=path.parent) or newpath
 
-		self.move_page(path, newpath, update_links=update_links)
+		self.move_page(path, newpath, update_links, callback)
 		if update_heading:
 			import zim.parsing
 			page = self.get_page(newpath)
@@ -1007,14 +1069,87 @@ class Notebook(gobject.GObject):
 
 		return newpath
 
-	def delete_page(self, path):
+	def delete_page(self, path, update_links=True, callback=None):
+		'''Delete a page. If 'update_links' is True pages linking to the
+		deleted page will be updated and the link are removed.
+		'''
+		# Collect backlinks
+		if update_links:
+			from zim.index import LINK_DIR_BACKWARD
+			backlinkpages = set()
+			for l in self.index.list_links(path, LINK_DIR_BACKWARD):
+				backlinkpages.add(l.source)
+
+			page = self.get_page(path)
+			if page.haschildren:
+				for child in self.index.walk(path):
+					for l in self.index.list_links(child, LINK_DIR_BACKWARD):
+						backlinkpages.add(l.source)
+
+		# actual delete
 		self.emit('delete-page', path)
 		store = self.get_store(path)
 		store.delete_page(path)
 		self.flush_page_cache(path)
 		path = Path(path.name)
+
+		# Update links to the deleted page tree
+		if update_links:
+			#~ print backlinkpages
+			total = len(backlinkpages)
+			for p in backlinkpages:
+				if p == path or p.ischild(path):
+					continue
+				page = self.get_page(p)
+				if callback: callback(page, total=total)
+				self._remove_links_in_page(page, path)
+				self.store_page(page)
+
+		# let everybody know what happened
 		self.emit('deleted-page', path)
 
+	def _remove_links_in_page(self, page, path):
+		logger.debug('Removing links in %s to %s', page, path)
+		tree = page.get_parsetree()
+		if not tree:
+			logger.warn('Page turned out to be empty: %s', page)
+			return
+
+		def walk_links(parent):
+			# Yields parent element, previous element and link element.
+			# we actually yield links in reverse order, so removal
+			# algorithm works for consequetive links as well.
+			children = parent.getchildren()
+			for i in range(len(children)-1, -1, -1):
+				if children[i].tag == 'link':
+					if i > 0: yield parent, children[i-1], children[i]
+					else: yield parent, None, children[i]
+				for items in walk_links(children[i]): # recurs
+					yield items
+
+		for parent, prev, element in walk_links(tree.getroot()):
+			try:
+				href = element.attrib['href']
+				type = link_type(href)
+				if type == 'page':
+					hrefpath = self.resolve_path(href, source=page)
+					#~ print 'LINK', hrefpath
+					if hrefpath == path \
+					or hrefpath.ischild(path):
+						# Remove the link
+						text = (element.text or '') + (element.tail or '')
+						if not prev is None:
+							prev.tail = (prev.tail or '') + text
+						else:
+							parent.text = (parent.text or '') + text
+						parent.remove(element)
+					else:
+						continue
+			except:
+				logger.exception('Error while removing link "%s"', href)
+
+		page.set_parsetree(tree)
+
 	def resolve_file(self, filename, path):
 		'''Resolves a file or directory path relative to a page. Returns a
 		File object. However the file does not have to exist.
@@ -1439,6 +1574,9 @@ class Page(Path):
 			else:
 				return hascontent
 
+	def exists(self):
+		return self.haschildren or self.hascontent
+
 	def get_parsetree(self):
 		'''Returns contents as a parsetree or None'''
 		assert self.valid, 'BUG: page object became invalid'
